@@ -111,9 +111,9 @@ int io_uring_register_files_update(struct io_uring *ring, unsigned off,
 {
 	liburing_sanitize_address(files);
 
-	struct io_uring_files_update up = {
+	struct io_uring_rsrc_update up = {
 		.offset	= off,
-		.fds	= (unsigned long) files,
+		.data	= (unsigned long) files,
 	};
 
 	return do_register(ring, IORING_REGISTER_FILES_UPDATE, &up, nr_files);
@@ -368,6 +368,11 @@ int io_uring_register_sync_cancel(struct io_uring *ring,
 	return do_register(ring, IORING_REGISTER_SYNC_CANCEL, reg, 1);
 }
 
+int io_uring_register_sync_msg(struct io_uring_sqe *sqe)
+{
+	return __sys_io_uring_register(-1, IORING_REGISTER_SEND_MSG_RING, sqe, 1);
+}
+
 int io_uring_register_file_alloc_range(struct io_uring *ring,
 					unsigned off, unsigned len)
 {
@@ -395,9 +400,9 @@ int io_uring_register_clock(struct io_uring *ring,
 	return do_register(ring, IORING_REGISTER_CLOCK, arg, 0);
 }
 
-int io_uring_clone_buffers_offset(struct io_uring *dst, struct io_uring *src,
-				  unsigned int dst_off, unsigned int src_off,
-				  unsigned int nr, unsigned int flags)
+int __io_uring_clone_buffers_offset(struct io_uring *dst, struct io_uring *src,
+				    unsigned int dst_off, unsigned int src_off,
+				    unsigned int nr, unsigned int flags)
 {
 	struct io_uring_clone_buffers buf = {
 		.src_fd		= src->ring_fd,
@@ -407,19 +412,40 @@ int io_uring_clone_buffers_offset(struct io_uring *dst, struct io_uring *src,
 		.nr		= nr,
 	};
 
-	if (src->int_flags & INT_FLAG_REG_REG_RING) {
+	if (flags & IORING_REGISTER_SRC_REGISTERED &&
+	    src->int_flags & INT_FLAG_REG_REG_RING) {
 		buf.src_fd = src->enter_ring_fd;
-		buf.flags |= IORING_REGISTER_SRC_REGISTERED;
 	} else {
 		buf.src_fd = src->ring_fd;
+		buf.flags &= ~IORING_REGISTER_SRC_REGISTERED;
 	}
 
 	return do_register(dst, IORING_REGISTER_CLONE_BUFFERS, &buf, 1);
 }
 
+int io_uring_clone_buffers_offset(struct io_uring *dst, struct io_uring *src,
+				  unsigned int dst_off, unsigned int src_off,
+				  unsigned int nr, unsigned int flags)
+{
+	return __io_uring_clone_buffers_offset(dst, src, dst_off, src_off, nr,
+						flags | IORING_REGISTER_SRC_REGISTERED);
+}
+
 int io_uring_clone_buffers(struct io_uring *dst, struct io_uring *src)
 {
-	return io_uring_clone_buffers_offset(dst, src, 0, 0, 0, 0);
+	return __io_uring_clone_buffers_offset(dst, src, 0, 0, 0, IORING_REGISTER_SRC_REGISTERED);
+}
+
+int __io_uring_clone_buffers(struct io_uring *dst, struct io_uring *src,
+			     unsigned int flags)
+{
+	return __io_uring_clone_buffers_offset(dst, src, 0, 0, 0, flags);
+}
+
+int io_uring_register_ifq(struct io_uring *ring,
+			  struct io_uring_zcrx_ifq_reg *reg)
+{
+	return do_register(ring, IORING_REGISTER_ZCRX_IFQ, reg, 1);
 }
 
 int io_uring_resize_rings(struct io_uring *ring, struct io_uring_params *p)
@@ -439,6 +465,7 @@ int io_uring_resize_rings(struct io_uring *ring, struct io_uring_params *p)
 
 	sq_head = ring->sq.sqe_head;
 	sq_tail = ring->sq.sqe_tail;
+	__sys_munmap(ring->sq.sqes, ring->sq.sqes_sz);
 	io_uring_unmap_rings(&ring->sq, &ring->cq);
 	memset(&ring->sq, 0, sizeof(ring->sq));
 	memset(&ring->cq, 0, sizeof(ring->cq));
@@ -476,3 +503,14 @@ int io_uring_register_region(struct io_uring *ring,
 {
 	return do_register(ring, IORING_REGISTER_MEM_REGION, reg, 1);
 }
+
+int io_uring_set_iowait(struct io_uring *ring, bool enable_iowait)
+{
+	if (!(ring->features & IORING_FEAT_NO_IOWAIT))
+		return -EOPNOTSUPP;
+	if (enable_iowait)
+		ring->int_flags &= ~INT_FLAG_NO_IOWAIT;
+	else
+		ring->int_flags |= INT_FLAG_NO_IOWAIT;
+	return 0;
+}
