@@ -110,6 +110,7 @@ static size_t params_cq_size(const struct io_uring_params *p, unsigned cqes)
 int io_uring_mmap(int fd, struct io_uring_params *p, struct io_uring_sq *sq,
 		  struct io_uring_cq *cq)
 {
+	size_t sqes_sz;
 	int ret;
 
 	sq->ring_sz = p->sq_off.array + p->sq_entries * sizeof(unsigned);
@@ -139,7 +140,14 @@ int io_uring_mmap(int fd, struct io_uring_params *p, struct io_uring_sq *sq,
 		}
 	}
 
-	sq->sqes = __sys_mmap(0, params_sqes_size(p, p->sq_entries),
+	sqes_sz = params_sqes_size(p, p->sq_entries);
+	sq->sqes_sz = (unsigned int) sqes_sz;
+	if (sq->sqes_sz != sqes_sz) {
+		ret = -EINVAL;
+		goto err;
+	}
+
+	sq->sqes = __sys_mmap(0, sq->sqes_sz,
 			      PROT_READ | PROT_WRITE, MAP_SHARED | MAP_POPULATE,
 			      fd, IORING_OFF_SQES);
 	if (IS_ERR(sq->sqes)) {
@@ -218,7 +226,7 @@ static int io_uring_alloc_huge(unsigned entries, struct io_uring_params *p,
 {
 	unsigned long page_size = get_page_size();
 	unsigned sq_entries, cq_entries;
-	size_t ring_mem, sqes_mem;
+	size_t sqes_size = 0, ring_mem, sqes_mem;
 	unsigned long mem_used = 0;
 	void *ptr;
 	int ret;
@@ -227,15 +235,14 @@ static int io_uring_alloc_huge(unsigned entries, struct io_uring_params *p,
 	if (ret)
 		return ret;
 
-	ring_mem = KRING_SIZE;
-
 	sqes_mem = params_sqes_size(p, sq_entries);
 	if (!(p->flags & IORING_SETUP_NO_SQARRAY))
 		sqes_mem += sq_entries * sizeof(unsigned);
 	sqes_mem = (sqes_mem + page_size - 1) & ~(page_size - 1);
 
-	ring_mem += sqes_mem + params_cq_size(p, cq_entries);
-	mem_used = ring_mem;
+	ring_mem = KRING_SIZE;
+	ring_mem += params_cq_size(p, cq_entries);
+	mem_used = sqes_mem + ring_mem;
 	mem_used = (mem_used + page_size - 1) & ~(page_size - 1);
 
 	/*
@@ -260,7 +267,8 @@ static int io_uring_alloc_huge(unsigned entries, struct io_uring_params *p,
 			buf_size = huge_page_size;
 			map_hugetlb = MAP_HUGETLB;
 		}
-		ptr = __sys_mmap(NULL, buf_size, PROT_READ|PROT_WRITE,
+		sqes_size = buf_size;
+		ptr = __sys_mmap(NULL, sqes_size, PROT_READ|PROT_WRITE,
 					MAP_SHARED|MAP_ANONYMOUS|map_hugetlb,
 					-1, 0);
 		if (IS_ERR(ptr))
@@ -268,6 +276,7 @@ static int io_uring_alloc_huge(unsigned entries, struct io_uring_params *p,
 	}
 
 	sq->sqes = ptr;
+	sq->sqes_sz = (unsigned int) sqes_size;
 	if (mem_used <= buf_size) {
 		sq->ring_ptr = (void *) sq->sqes + sqes_mem;
 		/* clear ring sizes, we have just one mmap() to undo */
@@ -285,7 +294,8 @@ static int io_uring_alloc_huge(unsigned entries, struct io_uring_params *p,
 					MAP_SHARED|MAP_ANONYMOUS|map_hugetlb,
 					-1, 0);
 		if (IS_ERR(ptr)) {
-			__sys_munmap(sq->sqes, 1);
+			if (sqes_size)
+				__sys_munmap(sq->sqes, sqes_size);
 			return PTR_ERR(ptr);
 		}
 		sq->ring_ptr = ptr;
@@ -330,7 +340,7 @@ int __io_uring_queue_init_params(unsigned entries, struct io_uring *ring,
 	if (fd < 0) {
 		if ((p->flags & IORING_SETUP_NO_MMAP) &&
 		    !(ring->int_flags & INT_FLAG_APP_MEM)) {
-			__sys_munmap(ring->sq.sqes, 1);
+			__sys_munmap(ring->sq.sqes, ring->sq.sqes_sz);
 			io_uring_unmap_rings(&ring->sq, &ring->cq);
 		}
 		return fd;
@@ -445,7 +455,7 @@ __cold void io_uring_queue_exit(struct io_uring *ring)
 	struct io_uring_cq *cq = &ring->cq;
 
 	if (!(ring->int_flags & INT_FLAG_APP_MEM)) {
-		__sys_munmap(sq->sqes, io_uring_sqes_size(ring));
+		__sys_munmap(sq->sqes, sq->sqes_sz);
 		io_uring_unmap_rings(sq, cq);
 	}
 
@@ -630,6 +640,7 @@ static struct io_uring_buf_ring *br_setup(struct io_uring *ring,
 			MAP_SHARED | MAP_POPULATE, ring->ring_fd, off);
 	if (IS_ERR(br)) {
 		*err = PTR_ERR(br);
+		io_uring_unregister_buf_ring(ring, bgid);
 		return NULL;
 	}
 
